# -*- coding: utf-8 -*-

import sys
import time

from PyQt4 import QtCore, QtGui


class ShootingView(QtGui.QGraphicsView):
    def __init__(self, parent=None):
        QtGui.QGraphicsView.__init__(self, parent)

    def resizeEvent(self, event):
        self.fitInView(self.scene().sceneRect(), QtCore.Qt.KeepAspectRatio)


class ShootingScene(QtGui.QGraphicsScene):
    def __init__(self, yawStart, yawEnd, pitchStart, pitchEnd, yawFov, pitchFov, yawCameraFov, pitchCameraFov, parent=None):
        QtGui.QGraphicsScene.__init__(self, parent)
        self._yawStart = yawStart
        self._yawEnd = yawEnd
        self._pitchStart = pitchStart
        self._pitchEnd = pitchEnd
        self._yawFov = yawFov
        self._pitchFov = pitchFov
        self._yawCameraFov = yawCameraFov
        self._pitchCameraFov = pitchCameraFov

        self._headCrosshair = CrosshairCusrsor()
        self._headCrosshair.setZValue(9999)
        self.addItem(self._headCrosshair)

        x = min(self._yawStart, self._yawEnd) - self._yawCameraFov / 2
        y = min(self._pitchStart, self._pitchEnd) - self._pitchCameraFov / 2
        w = self._yawFov
        h = self._pitchFov
        self.setSceneRect(x, y, w, h)

    def addPicture(self, yaw, pitch):
        picture = PictureItem(yaw, -pitch, self._yawCameraFov, self._pitchCameraFov)
        self.addItem(picture)

    def setHeadPosition(self, yaw, pitch):
        self._headCrosshair.setPos(yaw, -pitch)


class PictureItem(QtGui.QGraphicsItem):
    def __init__(self, yaw, pitch ,yawFov, pitchFov, parent=None):
        QtGui.QGraphicsItem.__init__(self, parent)
        self.setPos(yaw, pitch)
        self._yawFov = yawFov
        self._pitchFov = pitchFov

    def _computeBorderWidth(self):
        BORDER_WIDTH = 1
        xRatio = self.scene().views()[0].width() / self.scene().width()
        yRatio = self.scene().views()[0].height() / self.scene().height()
        return BORDER_WIDTH / min(xRatio, yRatio)

    def _computeRect(self):
        x = -self._yawFov / 2
        y = -self._pitchFov / 2
        w = self._yawFov
        h = self._pitchFov
        return x, y, w, h

    def boundingRect(self):
        x, y, w, h = self._computeRect()
        return QtCore.QRectF(x - self._computeBorderWidth() / 2, y - self._computeBorderWidth() / 2,
                             w + self._computeBorderWidth(), h + self._computeBorderWidth())

    def paint(self, painter, options, widget):
        x, y, w, h = self._computeRect()
        painter.fillRect(x, y, w, h, QtGui.QColor("white"))
        painter.setPen(QtGui.QPen(QtGui.QBrush(QtGui.QColor("black")), self._computeBorderWidth()))
        painter.drawRect(x, y, w, h)


class CrosshairCusrsor(QtGui.QGraphicsItemGroup):
    def __init__(self, parent=None):
        QtGui.QGraphicsItemGroup .__init__(self, parent)
        self._yawLine = QtGui.QGraphicsLineItem()
        self._yawLine.setLine(0, -1000, 0, 1000)
        self._yawLine.setPen(QtGui.QColor("red"))
        self.addToGroup(self._yawLine)
        self._pitchLine = QtGui.QGraphicsLineItem()
        self._pitchLine.setLine(-1000, 0, 1000, 0)
        self._pitchLine.setPen(QtGui.QColor("red"))
        self.addToGroup(self._pitchLine)

    def boundingRegion(self, transform):
        return QtGui.QRegion()


class Thread(QtCore.QThread):
    def __init__(self, scene):
        QtCore.QThread.__init__(self)
        self.__scene = scene
        self.__x = 0
        self.__y = 0

    def run(self):
        self.sleep(2)
        print "Start moving..."
        for i in xrange(100):
            self.__scene.setHeadPosition(self.__x, -self.__y)
            self.__x += 3.6
            self.__y += 1.8
            self.msleep(100)
        print "Stop moving"
        self.sleep(2)

def main():
    app = QtGui.QApplication(sys.argv)
    scene = ShootingScene(0, 360, 0, 180, 360, 180, 20, 30)
    for row in xrange(6):
        for col in xrange(18):
            scene.addPicture(col * 20, -row * 30)
    view = ShootingView(scene)
    thread = Thread(scene)
    QtCore.QObject.connect(thread, QtCore.SIGNAL("finished()"), app, QtCore.SLOT("quit()"))
    view.show()
    thread.start()
    app.exec_()


if __name__ == "__main__":
    main()
